#!/tvbin/tivosh
#
#  Copyright (C) 2003  Graham R. Cobb
#
#  SLIDESHOW For TiVo is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  SLIDESHOW For TiVo is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  IMPORTANT NOTES: TiVo is a registered trademark of Tivo, Inc.
#  This software is not created, endorsed, reviewed, approved or in any other
#  way associated with Tivo, Inc.
#
#  Thanks to embeem (mbm) for his insights into event handling and ioctls
#  Thanks to Jake B for his original slideshow program and his encouragement
#  to re-implement it in TCL
#

#
#	slideshow.tcl
#
# This TCL script uses jpegwriter to display the images from a directory
# on the TV screen.  It is a re-implementation of Jake B's slideshow program
# in TCL in the hope that this will make it more robust to changes in the event
# handling system.
#
# Invoke with --help to see the list of remote control keys
#

set about "SLIDESHOW For TiVo V1.1-1. Copyright (C) 2003 Graham R. Cobb"
puts $about

setpri fifo 1

source $tcl_library/tv/sendkey.tcl
# Add Thompson (UK Tivo) keys:
set evrc(T_aux) 37
set evrc(T_vcr) 38
set evrc(T_standby) 39

source $tcl_library/tv/log.tcl

# Partial list of context codes
set ctxc(tivoCentral) 2
set ctxc(playback) 6
set ctxc(liveTV) 6
set ctxc(nowPlaying) 7
set ctxc(nowPlayingDetail) 8
set ctxc(toDo) 32
set ctxc(searchByTitle) 33
set ctxc(standby) 109

# ioctl codes
set MPEG_OUTPUT_ENABLE 0x422
set MPEG_OSD_ON 0x3F3 

global wake_up img_index max_index timer_event time_delay currentContextG ctxc
set currentContextG 0
set img_index 0

proc key_callback {type subtype} {
	global EventData evrc wake_up time_delay timer_event currentContextG ctxc img_index max_index
# Only process keys if we are in standby
	if {$currentContextG == $ctxc(standby)} {
		binary scan $EventData IIIII keycode modifiers id timelow timehigh
	#	puts "Key: $keycode subtype=$subtype code=$keycode mod=$modifiers id=$id timelow=$timelow timehigh=$timehigh"
		if { $subtype == 0 } {
			switch $keycode \
			$evrc(replay) - \
			$evrc(up) {
				# Back one image
				prev_image
				set wake_up 1
				} \
			$evrc(play) - \
			$evrc(down) {
				# Next image
				next_image
				set wake_up 1
				} \
			$evrc(left) - \
			$evrc(thumbsdown) {
				# Exit
				exit
				} \
			$evrc(pause) {
				# Pause
				after cancel $timer_event
				} \
			$evrc(forward) {
				# Decrease time interval
				incr time_delay -10000
				set wake_up 1
				} \
			$evrc(reverse) {
				# Increase time interval
				incr time_delay 10000
				} \
			$evrc(catchup) {
				# Skip the rest of this directory
				set img_index $max_index
				set wake_up 1
				}
		}
	}
}

proc status_callback {type subtype} {
	global EventData currentContextG ctxc wake_up
	binary scan $EventData II context serialNo
	switch $subtype \
	$TmkEventMwStatus::CONTEXT_CHANGE  {
		set currentContextG $context
		if {$currentContextG == $ctxc(standby)} {
			set wake_up 1
		}
	}
}

proc image_timeout {} {
	global wake_up currentContextG ctxc
# Only process timer if we are in standby
	if {$currentContextG == $ctxc(standby)} {
		next_image
		set wake_up 1
	}
}

proc next_image {} {
	global img_index
	incr img_index
}

proc prev_image {} {
	global img_index
	if {[incr img_index -1] < 0} {set img_index 0}
}

event register $TmkEvent::EVT_REMOTEEVENT key_callback
event register $TmkEvent::EVT_MW_STATUS status_callback

# Option handling
set time_delay 10000
set jpg_options {}
for {set i 0} {$i < $argc} {incr i} {
	set opt [lindex $argv $i]
	switch -- $opt {
		"-?" -
		"--help" {
			puts "Display directories of JPEG images on Tivo screen in standby mode\n"
			puts "Remote control:"
			puts "\tPlay/Down-arrow\t\tNext image/Resume"
			puts "\tReplay/Up-arrow\t\tPrevious image"
			puts "\tThumbs down/Left\tExit"
			puts "\tPause\t\t\tPause"
			puts "\tFast Forward\t\tDecrease time interval (faster)"
			puts "\tRewind\t\t\tIncrease time interval (slower)"
			puts "\tCatchup\t\tNext directory"
			puts ""
			puts "Usage: $argv0 \[<options>\] <directory>..."
			puts "  -?, --help\t\tDisplay this help"
			puts "  -t, --time\t\tTime delay in seconds"
			puts "  -S, --standby\t\tTivo is already in standby mode, don't wait"
			puts "  all other options are passed through to jpegwriter and shown below\n"
			
			catch {exec jpegwriter --help} jpg_out
			puts $jpg_out
			exit
		}
		"-t" -
		"--time" {
			incr i
			set time_delay [expr [lindex $argv $i] * 1000]
		}
		"-S" -
		"--standby" {
			set currentContextG $ctxc(standby)
		}
		default {
			if {[string index $opt 0] == "-"} {
				lappend jpg_options $opt
			} else {
				lappend dir_list $opt
			}
		}
	}
}

# wait for the initial transition into standby mode
while {$currentContextG != $ctxc(standby)} {
	vwait wake_up
}

# Loop through the images
while 1 {
	foreach dir $dir_list {
		set images [glob $dir/*.{jpg,jpeg}]
		set max_index [llength $images]
		set img_index 0
		while {$img_index < $max_index} {
			# Only draw picture if we are in standby mode
			if {$currentContextG == $ctxc(standby)} {
				# In order to process events while we are waiting we must use vwait
				# So, we first set a timer event
				set timer_event [after $time_delay {image_timeout}]

				# Check the display is enabled
				if {![info exists mpeg_dev]} {
					set mpeg_dev [open "/dev/mpeg0v" RDWR]
				}
				ioctl $mpeg_dev $MPEG_OUTPUT_ENABLE -param 1
				ioctl $mpeg_dev $MPEG_OSD_ON

				# Now draw the picture
				set jpg_file [lindex $images $img_index]
				# puts $jpg_file
				# Ignore any errors, but display them
				set jpgw_status [catch {
					# Note - the "eval/concat" is needed on the following line to cause the $jpg_options
					# to be split into separate arguments, not a single argument
					eval [concat exec jpegwriter $jpg_options $jpg_file]
					} jpgw_output]
				if {$jpgw_status != 0} {puts $jpgw_output}
			}

			# Now wait either for the timer or for a key-press
			vwait wake_up

			# Just in case it wasn't the timer event which fired, cancel it
			after cancel $timer_event
		}
	}
}
